/*
 * Copyright (C) 2011 SHARP Corporation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package jp.co.sharp.android.SampleStereo3DWallpaper;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.os.Handler;
import android.view.SurfaceHolder;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.PackageManager;
import android.content.res.*;
import jp.co.sharp.android.stereo3dlcd.*;
import jp.co.sharp.android.SampleStereo3DWallpaper.R;

public class SampleStereo3DWallpaper extends StereoWallpaperService {

    /* 2D/3D切替用のIntent Action　 */
    private static final String ACTION_CHANGE_PARAM_3D = "jp.co.sharp.android.wallpaper3d.ACTION_CHANGE_PARAM3D";
    private float mDepthF = 1.0f; /* 奥行き設定情報 Float */
    Context mContext; /* コンテキスト情報 */

    /*
     * Feature定義をチェックし、奥行き設定情報を取得する。
     */
    @Override
    public void onCreate() {
        super.onCreate();
        mContext = this;
        getDepth();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
    }

    public Engine onCreateEngine() {
        return new StereoEngine(getResources());
    }

    public class StereoEngine extends Engine {
        private final Handler handler = new Handler(); /* 描画用のハンドラー */
        private Bitmap mImage; /* 画像データ（2D表示用） */
        private Bitmap mImageHalf; /* 横幅半分の画像データ（Stereo3D用） */
        private Bitmap mBackImage = null, mBackImage3D = null; /* 背景画像データ */
        private int mPosX = 0; /* 画像表示のX座標 */
        private int mPosY = 0; /* 画像表示のY座標 */
        private int mVectorX = 10; /* 画像表示の移動量（X座標） */
        private int mVectorY = 10; /* 画像表示の移動量（Y座標） */
        private boolean mVisible; /* 可視状態 */
        private int mWidth; /* Surfaceの横サイズ */
        private int mHeight; /* Surfaceの縦サイズ */
        private BroadcastReceiver mReceiver;/* 2D/3D切替イベント取得用レシーバー */
        private boolean mStereo3D = false; /* Stereo 3D 状態 */
        private int mDepth = 0; /* 奥行き情報 */

        /*
         * リソースから画像データを取得し、Stereo3D表示用に横幅1/2の画像を生成する。
         */
        public StereoEngine(Resources r) {
            mImage = BitmapFactory.decodeResource(r, R.drawable.android);
            mImageHalf = Bitmap.createScaledBitmap(mImage,
                    mImage.getWidth() / 2, mImage.getHeight(), true);
        }

        /*
         * 描画用のスレッド
         */
        private final Runnable drawWallpaperThread = new Runnable() {
            public void run() {
                drawWallpaper();
            }
        };

        /*
         * 3D表示設定の情報を取得し、2D/3D切替設定を行う。 また、2D/3D切替のイベント受信処理を登録する。
         */
        @Override
        public void onCreate(SurfaceHolder surfaceHolder) {
            super.onCreate(surfaceHolder);

            if (isStereoMode()) { /* ★Addon用API使用箇所★ */
                mStereo3D = true;
                setStereoView(true); /* ★Addon用API使用箇所★ */
            } else {
                mStereo3D = false;
                setStereoView(false); /* ★Addon用API使用箇所★ */
            }

            mReceiver = new BroadcastReceiver() {

                @Override
                public void onReceive(Context context, Intent intent) {
                    if ((intent.getAction().equals(ACTION_CHANGE_PARAM_3D))) {
                        int type = intent.getIntExtra(
                                "Extra_3D_WallPaper_SettingType", 0);
                        synchronized (mContext) {
                            if (type == StereoWallpaperService.Engine.STEREO_MODE_ON) {
                                mStereo3D = true;
                                setStereoView(true); /* ★Addon用API使用箇所★ */
                            } else {
                                mStereo3D = false;
                                setStereoView(false); /* ★Addon用API使用箇所★ */
                            }
                        }
                    }
                }
            };
            IntentFilter filter = new IntentFilter(ACTION_CHANGE_PARAM_3D);
            registerReceiver(mReceiver, filter);
        }

        /*
         * 描画用のスレッドをとめる
         */
        @Override
        public void onDestroy() {
            super.onDestroy();
            handler.removeCallbacks(drawWallpaperThread);
            unregisterReceiver(mReceiver);
        }

        /*
         * 可視状態になった場合、奥行き設定情報を再取得し、表示する。 不可視状態になった場合、描画用のスレッドをとめる。
         */
        @Override
        public void onVisibilityChanged(boolean mVisible) {
            this.mVisible = mVisible;
            if (mVisible) {
                getDepth();
                drawWallpaper();
            } else {
                handler.removeCallbacks(drawWallpaperThread);
            }
        }

        /*
         * Surfaceのサイズを取得し、描画処理を行う。
         */
        @Override
        public void onSurfaceChanged(SurfaceHolder holder, int format,
                int mWidth, int mHeight) {
            super.onSurfaceChanged(holder, format, mWidth, mHeight);
            this.mWidth = mWidth;
            this.mHeight = mHeight;
            if (mPosX > mWidth)
                mPosX = mWidth;
            if (mPosY > mHeight)
                mPosY = mHeight;
            drawWallpaper();
        }

        @Override
        public void onSurfaceCreated(SurfaceHolder holder) {
            super.onSurfaceCreated(holder);
        }

        /*
         * 描画用のスレッドをとめる。
         */
        @Override
        public void onSurfaceDestroyed(SurfaceHolder holder) {
            super.onSurfaceDestroyed(holder);
            mVisible = false;
            handler.removeCallbacks(drawWallpaperThread);
        }

        /*
         * 描画処理を行う。
         */
        @Override
        public void onOffsetsChanged(float xOffset, float yOffset, float xStep,
                float yStep, int xPixels, int yPixels) {
            drawWallpaper();
        }

        /*
         * 背景画像を描画する。
         * 必要に応じてリソースから画像データを取り出す。
         */
        private void drawBack(Canvas canvas) {
            Rect dstRect = new Rect(0, 0, mWidth, mHeight);
            if (mStereo3D == true) {
                if (mBackImage3D == null) {
                    Resources res = mContext.getResources();
                    mBackImage3D = BitmapFactory.decodeResource(res, R.drawable.welcome3d);
                }
                Rect srcRect = new Rect(0, 0,
                        mBackImage3D.getWidth(), mBackImage3D.getHeight());
                canvas.drawBitmap(mBackImage3D, srcRect, dstRect, null);
            } else {
                if (mBackImage == null) {
                    Resources res = mContext.getResources();
                    mBackImage = BitmapFactory.decodeResource(res, R.drawable.welcome);
                }
                Rect srcRect = new Rect(0, 0,
                        mBackImage.getWidth(), mBackImage.getHeight());
                canvas.drawBitmap(mBackImage, srcRect, dstRect, null);
            }
        }
        
        /*
         * Canvasに対する描画処理。 
         * 3D表示のモードでは、SidebySide(half)を描画する。 次回描画タイミングの登録も行う。
         */
        private void drawWallpaper() {

            synchronized (mContext) {
                SurfaceHolder holder = getSurfaceHolder();
                Canvas canvas = holder.lockCanvas();
                if (canvas == null)
                    return;

                drawBack(canvas);

                int x = mPosX - 52;
                int y = mPosY - 52;

                mDepth++;
                if (mDepth > 100)
                    mDepth = 0;
                int d = mDepth / 5;
                if (mDepth > 50)
                    d = (100 - mDepth) / 5;

                d *= mDepthF;

                if (mStereo3D) {
                    /* 3D表示モードの場合 */
                    Paint p = new Paint();
                    int xTextOffset = mImageHalf.getWidth() / 2 - 8;
                    int yTextOffset = mImageHalf.getHeight() / 2;
                    p.setColor(Color.BLACK);
                    canvas.clipRect(0, 0, mWidth / 2, mHeight);
                    canvas.drawBitmap(mImageHalf, x / 2, y, null);
                    canvas.drawText("" + d, x / 2 + xTextOffset, y + yTextOffset, p);
                    canvas.restore();
                    canvas.clipRect(mWidth / 2, 0, mWidth, mHeight);
                    canvas.drawBitmap(mImageHalf, mWidth / 2 + x / 2 + d, y, null);
                    canvas.drawText("" + d, mWidth / 2 + x / 2 + d + xTextOffset, y
                            + yTextOffset, p);
                } else {
                    /* 2D表示モードの場合 */
                    canvas.drawBitmap(mImage, x, y, null);
                }

                holder.unlockCanvasAndPost(canvas);
            }

            if (mPosX < 0 || mWidth < mPosX)
                mVectorX = -mVectorX;
            if (mPosY < 0 || mHeight < mPosY)
                mVectorY = -mVectorY;
            mPosX += mVectorX;
            mPosY += mVectorY;

            /* 次回描画タイミングの登録 */
            handler.removeCallbacks(drawWallpaperThread);
            if (mVisible)
                handler.postDelayed(drawWallpaperThread, 100);
        }

    }

    /*
     * 3D飛び出し設定を取得し、奥行き設定情報に変換する。
     */
    private void getDepth() {

        Parallax p = new Parallax();
        mDepthF = 2.0f - p.get3dDepthSettingsFl(); /* ★Addon用API使用箇所★ */
    }

}